//===========================================================================
//  $Name: cflowd-2-1-b1 $
//  $Id: CflowdInterfaceMatrix.hh,v 1.2 1998/09/21 18:54:27 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, cflowd++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, cflowd++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for cflowd++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  cflowd++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with cflowd++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _CFLOWDINTERFACEMATRIX_HH_
#define _CFLOWDINTERFACEMATRIX_HH_

extern "C" {
#include <sys/types.h>

#include "caida_t.h"
}

#include <map>
#include <iostream.h>

#include "CflowdInterfaceMatrixKey.hh"
#include "CflowdInterfaceMatrixTrafficCounter.hh"
#include "CflowdRawFlow.hh"

typedef  map<CflowdInterfaceMatrixKey, CflowdInterfaceMatrixTrafficCounter, less<CflowdInterfaceMatrixKey> >   CflowdInterfaceMatrixMap_t;

//---------------------------------------------------------------------------
//  class CflowdInterfaceMatrix : public CflowdInterfaceMatrixMap_t
//---------------------------------------------------------------------------
//  This class is used to contain interface matrix traffic data in cflowd++.
//  We use a CflowdInterfaceMatrixKey as a key, and a
//  CflowdInterfaceMatrixTrafficCounter to hold packet and byte counters.
//  Note we just inherit from an instantiation of an STL map, then just
//  add a few new member functions.
//---------------------------------------------------------------------------
class CflowdInterfaceMatrix : public CflowdInterfaceMatrixMap_t
{
public:
  typedef CflowdInterfaceMatrixMap_t::iterator        iterator;
  typedef CflowdInterfaceMatrixMap_t::const_iterator  const_iterator;

  //-------------------------------------------------------------------------
  //                 int AddFlow(const CflowdRawFlow & flow)
  //.........................................................................
  //  Adds the data from a raw flow to the interface matrix.  Returns
  //  0 on success, -1 on failure (for example, the flow doesn't
  //  contain the information needed).
  //-------------------------------------------------------------------------
  int AddFlow(const CflowdRawFlow & flow);
  
  //-------------------------------------------------------------------------
  //                      istream & read(istream & is)
  //.........................................................................
  //  Reads the contents of an interface matrix from an istream.
  //  Returns the istream.
  //-------------------------------------------------------------------------
  istream & read(istream & is);
  
  //-------------------------------------------------------------------------
  //                            int read(int fd)
  //.........................................................................
  //  Reads the contents of an interface matrix from a file descriptor.
  //  Returns the number of bytes read on success, -1 on failure.
  //-------------------------------------------------------------------------
  int read(int fd);
  
  //-------------------------------------------------------------------------
  //                   ostream & write(ostream & os) const
  //.........................................................................
  //  Writes the contents of an interface matrix to an ostream.
  //  Returns the ostream.
  //-------------------------------------------------------------------------
  ostream & write(ostream & os) const;
  
  //-------------------------------------------------------------------------
  //                         int write(int fd) const
  //.........................................................................
  //  Writes the contents of an interface matrix to a file descriptor.
  //  Returns the number of bytes written on success, -1 on failure.
  //-------------------------------------------------------------------------
  int write(int fd) const;
};

#endif  // _CFLOWDINTERFACEMATRIX_HH_
