//===========================================================================
//  $Name: cflowd-2-1-b1 $
//  $Id: CflowdCollectorMap.hh,v 1.2 1998/09/21 17:48:32 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, cflowd++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, cflowd++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for cflowd++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  cflowd++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with cflowd++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _CFLOWDCOLLECTORMAP_HH_
#define _CFLOWDCOLLECTORMAP_HH_
extern "C" {
#include <sys/types.h>

#include "caida_t.h"
}

#include <map>

#include "CflowdCollector.hh"

typedef map<ipv4addr_t,CflowdCollector *,less<ipv4addr_t> > _CflowdCollectorMap_t;

//---------------------------------------------------------------------------
//  class CflowdCollectorMap : public _CflowdCollectorMap_t             
//---------------------------------------------------------------------------
//  This class is used to hold CflowdCollector objects in cflowd.
//  We just inherit from an STL map of CflowdCollector pointers,
//  keyed by IP address.
//---------------------------------------------------------------------------
class CflowdCollectorMap : public _CflowdCollectorMap_t
{
public:
  typedef _CflowdCollectorMap_t::iterator        iterator;
  typedef _CflowdCollectorMap_t::const_iterator  const_iterator;
  
  //-------------------------------------------------------------------------
  //                              void Clear()
  //.........................................................................
  //  Clears the map.
  //-------------------------------------------------------------------------
  void Clear();
};

#endif  // _CFLOWDCOLLECTORMAP_HH_
